from __future__ import print_function
import CMap2D
import numpy as np
import matplotlib.pyplot as plt
from timeit import default_timer as timer

from map2d import gridshow
from pyniel.testing_tools.testgen import load_test_case

import unittest

class TestStringMethods(unittest.TestCase):

    def test_office_full_coarse(self):
        cmap2d = CMap2D.CMap2D(".", "office_full", silent=True)
        coarse = cmap2d.as_coarse_map2d(n=3)
        test_data = load_test_case(folder="test_data", case_name="office_full_coarse")
        self.assertTrue(np.allclose(test_data['outputs'], coarse.occupancy()))

    def test_office_full_coarse_as_sdf(self):
        cmap2d = CMap2D.CMap2D(".", "office_full", silent=True)
        coarse = cmap2d.as_coarse_map2d().as_coarse_map2d().as_coarse_map2d()
        test_data = load_test_case(folder="test_data", case_name="office_full_coarse_as_sdf")
        self.assertTrue(np.allclose(test_data['outputs'], coarse.as_sdf()))

    def test_default_dijkstra_16(self):
        cmap2d = CMap2D.CMap2D(".", "tests", silent=True)
        test_data = load_test_case(folder="test_data", case_name="default_dijkstra_16")
        args = test_data['args']
        kwargs = test_data['kwargs']
        self.assertTrue(np.allclose(test_data['outputs'], cmap2d.dijkstra(*args, **kwargs)))

    def test_default_dijkstra_32(self):
        cmap2d = CMap2D.CMap2D(".", "tests", silent=True)
        test_data = load_test_case(folder="test_data", case_name="default_dijkstra_32")
        args = test_data['args']
        kwargs = test_data['kwargs']
        self.assertTrue(np.allclose(test_data['outputs'], cmap2d.dijkstra(*args, **kwargs)))

    def test_coord_transforms(self):
        cmap2d = CMap2D.CMap2D(".", "default", silent=True)
        ij = cmap2d.xy_to_ij(np.array([[0, 0], [1,1]], dtype=np.float32))
        cmap2d.ij_to_xy(ij.astype(np.float32))

    def test_dijkstra(self):
        cmap2d = CMap2D.CMap2D(".", "default", silent=True)
        # setting an invalid / masked point as goal should not segfault
        cmap2d.dijkstra(np.array([31,36]))

if __name__ == '__main__':
    unittest.main()



import numpy as np
from timeit import default_timer as timer


def make_circular_index(pixel_radius, sample_factor=10):
    """ Creates a circular index around the point at index 0, 0
    This method is sample-based and therefore not precise. 
    Solution is found by sampling at regular interval along the circle
    and storing the resulting points in 2d grid of resolution 1.
    Points are returned in order, forming a 2d grid circular path."""
    px_r = pixel_radius # radius of the circle in pixels

    # ensures sample_factor samples per angular pixel.
    kNSamples = max(12, 2 * np.pi * px_r * sample_factor) 

    theta = np.arange(0, kNSamples) * 2 * np.pi / kNSamples
    x = px_r * np.cos(theta)
    y = px_r * np.sin(theta)
    i = np.round(x).astype(int)
    j = np.round(y).astype(int)
    ij = np.vstack([i,j])
    unique_id = np.sort(np.unique(ij, axis=1, return_index=True)[1])
    i = ij[0, unique_id]
    j = ij[1, unique_id]
    return i, j

def index_to_grid(i, j, gridsize=None):
    """ Converts the circle from index format to 2d-grid format """
    size = gridsize
    if gridsize is None:
        size_i = max(i) - min(i) + 3
        size_j = max(j) - min(j) + 3
        size = max(size_i, size_j)
    mid = int((size - 1) / 2)
    grid = np.zeros((size, size))
    grid[(i+mid, j+mid)] = 1
    return grid

class CircularIndexCreator(object):
    """ Allows creating circular indices for given radii,
    and stores the results in a lookup table to avoid repeat computation """
    def __init__(self):
        self.px_r_archive = {}
        self.indices_archive = {}

    def make_circular_index(self, radius, resolution=1.):
        r = radius # [m]
        res = resolution # [m / px]
        px_r = radius / resolution # pixel-radius of circle [px]

        if px_r in self.px_r_archive:
            i, j = self.px_r_archive[px_r]
            return i, j

        i, j = make_circular_index(px_r)
        # store result in lookup table
        self.px_r_archive[px_r] = (i, j)
        return i, j

if __name__ == "__main__":
    from matplotlib import pyplot as plt
    N_CIRCLES = 1000
    PLOT_RADII = False
    PLOT_EVOLUTION = True
    px_radii = np.linspace(0, 100, N_CIRCLES)
    cc = CircularIndexCreator()
    circle_lengths = []
    t_st = timer()
    for px_r in px_radii:
        i, j = cc.make_circular_index(px_r, 1)
        circle_lengths.append(len(i))
    t_end = timer()
    print("First-time computation of {} circles took {} seconds.".format(N_CIRCLES, t_end - t_st))
    t_st = timer()
    for px_r in px_radii:
        i, j = cc.make_circular_index(px_r, 1)
    t_end = timer()
    print("Second-time computation of {} circles took {} seconds.".format(N_CIRCLES, t_end - t_st))
    if PLOT_RADII:
        plt.figure()
        for px_r in px_radii:
            i, j = cc.make_circular_index(px_r, 1)
            plt.plot(i, j, '-x')
        plt.show()
        plt.figure()
        plt.plot(px_radii, circle_lengths, '-x')
        plt.show()
    if PLOT_EVOLUTION:
        print("Plotting evolution of circles with radius")
        N = 20
        px_radii = np.logspace(np.log(0.5),np.log10(N-1),200)
        for px_r in px_radii:
            i, j = cc.make_circular_index(px_r, 1)
        plt.ion()
        plt.figure()
        for px_r in px_radii:
            plt.gca().clear()
            i, j = cc.make_circular_index(px_r, 1)
            grid = index_to_grid(i, j, 2*N+1)
            plt.imshow(1-grid, cmap='gray')
            plt.gca().add_patch(plt.Circle((N,N), radius=px_r, color='white', fill=False))
            plt.pause(0.005)
        plt.ioff()



import numpy
from setuptools import setup
from Cython.Build import cythonize

with open("README.md", 'r') as f:
    long_description = f.read()

setup(
    name="pymap2d",
    description='Tools for 2D maps',
    author='Daniel Dugas',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://github.com/danieldugas/pymap2d",
    version='0.1.12',
    py_modules=['map2d', 'pose2d', 'circular_index', 'map2d_ros_tools'],
    ext_modules=cythonize("CMap2D.pyx", annotate=True),
    install_requires=['pyyaml', 'numpy'],
    include_dirs=[numpy.get_include()],
)
